-----------------------------------------------------------------------------------------
-- Start Activity
-----------------------------------------------------------------------------------------

function PrisonEscapeMission:StartActivity()
    print("START! -- PrisonEscapeMission:StartActivity()!");
    --These 4 seem to be in every mission.  Not sure what they do, but I don't want to mess with them just in case they're important. 
    self.FightStage = { NOFIGHT = 0, DEFENDING = 1, ATTACK = 2, FIGHTSTAGECOUNT = 3 };
    self.AreaTimer = Timer();
    self.StepTimer = Timer();
    self.SpawnTimer = Timer();
    self.ExplosionTimer = Timer();
    --This is used in any mission to determine the current stage.  NOFIGHT means that enemies won't spawn.
    self.CurrentFightStage = self.FightStage.NOFIGHT;
    --The current checkpoint.  0 = Prison cell.  1 = Next to the first ally.  2 = Below the signal jammer's tube.  3 = Below the third ally.
    checkpoint = 0;
    --All the enterable zones that are used later.  Any zone simply used to find an object, for instance "Prison Doors 1" are defined later because they are temporary.
    self.check1 = SceneMan.Scene:GetArea("Checkpoint 1");
    self.check2 = SceneMan.Scene:GetArea("Checkpoint 2");
    self.check3 = SceneMan.Scene:GetArea("Checkpoint 3");
    self.dropzone = SceneMan.Scene:GetArea("Dig Field");
    self.gensect = SceneMan.Scene:GetArea("Generator Room");
    --The current character being played as.
    playerActor = nil;
    --This is to make sure nothing happens until the level has started.
    self.timer = 0;
    --How many lives the player has.  They have one less at the start because the first time they are spawned, a life is lost.
    if self.Difficulty <= GameActivity.CAKEDIFFICULTY then
	    self.Lives = 7;
	elseif self.Difficulty <= GameActivity.EASYDIFFICULTY then
	    self.Lives = 6;
	elseif self.Difficulty <= GameActivity.MEDIUMDIFFICULTY then
	    self.Lives = 4;
	elseif self.Difficulty <= GameActivity.HARDDIFFICULTY then
	    self.Lives = 3;
	elseif self.Difficulty <= GameActivity.NUTSDIFFICULTY then
	    self.Lives = 2;
	elseif self.Difficulty <= GameActivity.MAXDIFFICULTY then
	    self.Lives = 1;
	end
    --Changes the alternate messages to display at the end.
    self.msg2 = 0;
    --Used at the end to determine if an explosion should happen or not.
    self.explode = 0;
    --This stores if the activity is over.
    varend = 0;
    --If the first objective is complete.
    self.obj1 = 0;
    --Whether or not rocks have fallen down the tube near the generator yet.
    self.rocksdropped = 0;
    --The last playerActor before the ship comes down.
    oldactor = nil;
    --Last position of playerActor.
    lastpos = nil;
    --Whether or not the prisoner of X number are free.
    prisoner1free = 0;
    prisoner2free = 0;
    prisoner3free = 0;
    --The prisoners.
    prisoner1 = nil;
    prisoner2 = nil;
    prisoner3 = nil;
    --Whether or not the signal jammer is still there.
    jammergone = 0;
    --Whether or not the generator is going supernova.
    generatornova = 0;
    --How long before the explosion happens from when the generator is broken.  Divided by 1000 = seconds before the explosion.
    self.boomtimer = 120000;
    --Whether or not the ship at the end has come down.
    shipdropped = 0;
    --This keeps track of all the unlocked weapons.
    --See the function "respawn" for more information on how these are used.
    --1 = Stone.
    stone = 0;
    --1 = Pistol.  2 = Auto Pistol.  3 = Heavy Pistol.
    pistol = 0;
    --1 = YAK47.  2 = Assault Rifle.  3 = Gatling Gun.
    auto = 0;
    --1 = Sniper Rifle.
    sniper = 0;
    --1 = Shotgun.  2 = Auto Shotgun.
    shotgun = 0;
    --1 = Missile Launcher.
    cannon = 0;
    -- 1 = Flamer.
    flame = 0;

    --------------------------
    -- Set up teams

    -- Team 2 is always CPU
    self.CPUTeam = Activity.TEAM_2;
	ActivityMan:GetActivity():SetTeamFunds(0,0);
    
    --------------------------
    -- Set up players
    for player = 0, self.PlayerCount - 1 do
	--Set the brain to playerActor.  The only reason we do this is that the game crashes without a brain.  Even if the playerActor dies, it won't matter, though.
	playerActor = respawn();
	--Create a brain.
	local brain = CreateActor("Brain Case");
	MovableMan:AddActor(brain);
	self:SetPlayerBrain(brain,player);
	--Make the brain invisible.
	brain.Scale = 0;
	self:SetLandingZone(playerActor.Pos, player);
    end
	

    self.AreaTimer:Reset();
    self.StepTimer:Reset();
    self.SpawnTimer:Reset();

    --Play the music.
    AudioMan:ClearMusicQueue();
    AudioMan:PlayMusic("Prison Escape Ep1.rte/Music/FarFromDead.ogg", -1, -1);
end

--Make a blinding explosion at the end.
function createexplosion()
	local emitter = CreateMOSParticle("Mega Glow");
	--The camera can't go higher than the game's resolution divided by two, because otherwise it would leave the visible area.  The player can, though, and this messes up offsets if the ship just returned.  So...
	if shipdropped == 1 then
		--If the ship has been deployed, we assume that we're looking at the surface, so we set the glow's Y coordinate to 400 pixels down.
		emitter.Pos = Vector(lastpos.X,400);
	else
		--Otherwise, it's likely the player is still below the surface, so we just add 200 on to whatever they player's coordinates are.
		emitter.Pos = Vector(lastpos.X,lastpos.Y+200);
	end
	--Add the particle to the screen, making the glow effect show up.
	MovableMan:AddParticle(emitter);
	--Stop all the music and only play the explosion so the effect has more impact.
	AudioMan:ClearMusicQueue();
	AudioMan:PlayMusic("Prison Escape Ep1.rte/Explode.wav", 0, -1);
    
    glowpos = SceneMan:GetScrollTarget(Activity.PLAYER_1);
end

--Check the player's inventory and set the respective variables so we can respawn it with those guns.
function checkinv()
--Make sure the actor exists, or some errors will occur.
	if ((MovableMan:IsActor(playerActor))) then
		if playerActor:HasObject("Stone") then
			stone = 1;
		end
		if playerActor:HasObject("Pistol") and pistol < 1 then
			pistol = 1;
		end
		if playerActor:HasObject("Auto Pistol") and pistol < 2 then
			pistol = 2;
		end
		if playerActor:HasObject("Heavy Pistol") then
			pistol = 3;
		end		
		if playerActor:HasObject("YAK47") and auto < 1 then
			auto = 1;
		end
		if playerActor:HasObject("Assault Rifle") and auto < 2 then
			auto = 2;
		end
		if playerActor:HasObject("Gatling Gun") then
			auto = 3;
		end
		if playerActor:HasObject("Sniper Rifle") then
			sniper = 1;
		end
		if playerActor:HasObject("Shotgun") and shotgun < 1 then
			shotgun = 1;
		end
		if playerActor:HasObject("Auto Shotgun") then
			shotgun = 2;
		end
		if playerActor:HasObject("Missile Launcher") then
			cannon = 1;
		end
		if playerActor:HasObject("Flamer") then
			flame = 1;
		end
	end
end

--Spawn the player at the last achieved checkpoint with all weapons so far unlocked (except grenades, that's overkill)
function respawn()

--Make the player and set all its variables.
	pl = CreateAHuman("Dafred","Prison Escape Ep1.rte");
	pl.AIMode = Actor.AIMODE_SENTRY;
	pl.Team = 0;

--Get the list of weapons the player has.
	local wep0 = nil;
	local wep1 = nil;
	local wep2 = nil;
	local wep3 = nil;
	local wep4 = nil;
	local wep5 = nil;
	local wep6 = nil;

	shovel = CreateHDFirearm("Shovel");
	if stone == 1 then
		wep0 = CreateTDExplosive("Stone");
	end
	if pistol == 1 then
		wep1 = CreateHDFirearm("Pistol");
	end
	if pistol == 2 then
		wep1 = CreateHDFirearm("Auto Pistol");
	end
	if pistol == 3 then
		wep1 = CreateHDFirearm("Heavy Pistol");
	end
	if auto == 1 then
		wep2 = CreateHDFirearm("YAK47");
	end
	if auto == 2 then
		wep2 = CreateHDFirearm("Assault Rifle");
	end
	if auto == 3 then
		wep2 = CreateHDFirearm("Gatling Gun");
	end
	if sniper == 1 then
		wep3 = CreateHDFirearm("Sniper Rifle");
	end
	if shotgun == 1 then
		wep4 = CreateHDFirearm("Shotgun");
	end
	if shotgun == 2 then
		wep4 = CreateHDFirearm("Auto Shotgun");
	end
	if cannon == 1 then
		wep5 = CreateHDFirearm("Missile Launcher");
	end
	if flame == 1 then
		wep6 = CreateHDFirearm("Flamer");
	end

--Add the weapons to the player's inventory if it has them.
	pl:AddInventoryItem(shovel);
	if wep0 ~= nil then
		pl:AddInventoryItem(wep0);
	end
	if wep1 ~= nil then
		pl:AddInventoryItem(wep1);
	end
	if wep2 ~= nil then 
		pl:AddInventoryItem(wep2);
	end
	if wep3 ~= nil then 
		pl:AddInventoryItem(wep3);
	end
	if wep4 ~= nil then 
		pl:AddInventoryItem(wep4);
	end
	if wep5 ~= nil then 
		pl:AddInventoryItem(wep5);
	end
	if wep6 ~= nil then 
		pl:AddInventoryItem(wep6);
	end

--Set the player's position
	if checkpoint == 0 then
		pl.Pos = Vector(540,756);
	end
	if checkpoint == 1 then
		pl.Pos = Vector(1139,420);
	end
	if checkpoint == 2 then
		pl.Pos = Vector(469,420);
	end
	if checkpoint == 3 then
		pl.Pos = Vector(108,804);
	end

--FINALLY spawn the player.
	MovableMan:AddActor(pl);

--Also, return the player variable so that we know which actor it is.
	return pl;
end

-----------------------------------------------------------------------------------------
-- Pause Activity
-----------------------------------------------------------------------------------------

function PrisonEscapeMission:PauseActivity(pause)
    print("PAUSE! -- PrisonEscapeMission:PauseActivity()!");
end

-----------------------------------------------------------------------------------------
-- End Activity
-----------------------------------------------------------------------------------------

function PrisonEscapeMission:EndActivity()
	print("END! -- PrisonEscapeMission:EndActivity()!");
	--End the game.
	varend = 1;
	--These are all done after the mission ends, because otherwise they are replaced with "Lost control of remote body!" or "Returned craft and contents for X funds."
	if self.WinnerTeam == Activity.TEAM_1 then
		self:ResetMessageTimer(Activity.TEAM_1);
        FrameMan:ClearScreenText(Activity.TEAM_1);
		FrameMan:SetScreenText("I can't believe you made it! Even so - there's trouble at the base. Get back as soon as you can. (MISSION COMPLETE)", Activity.TEAM_1, 0, 5000, false);
	elseif self.msg2 == 1 then
		self:ResetMessageTimer(Activity.TEAM_1);
        FrameMan:ClearScreenText(Activity.TEAM_1);
		FrameMan:SetScreenText("Oh no... You left somebody behind. (MISSION FAILED)", Activity.TEAM_1, 0, 5000, false);
	elseif self.msg2 == 2 then
		self:ResetMessageTimer(Activity.TEAM_1);
        FrameMan:ClearScreenText(Activity.TEAM_1);
		FrameMan:SetScreenText("Oh god no... That was our last ship...", Activity.TEAM_1, 0, 5000, false);
		
		AudioMan:ClearMusicQueue();
		AudioMan:PlayMusic("Base.rte/Music/dBSoundworks/udiedfinal.ogg", 0, -1);
	end
	--Create the explosion.
	if self.explode == 1 then
		createexplosion();
	end
end

-----------------------------------------------------------------------------------------
-- Update Activity-----------------------------------------------------------------------------------------

function PrisonEscapeMission:UpdateActivity()
	if varend ~= 1 then
		-- Odd need for a delay here, probably to allow actors to spawn
		if self.timer == 5 then
			--Crumple the dead skeleton prisoner.
			for actor in MovableMan.Actors do
				if actor.PresetName == "Skeleton" then
					actor.Health = 0;
					actor.Vel = Vector(-1,2);
				end
			end

			--Store the prisoners as a variable.
			for actor in MovableMan.Actors do
				if actor.PresetName == "Ronin Soldier 4 Nopack" then
					prisoner1 = actor;
				end

				if actor.PresetName == "Ronin Soldier 5 Nopack" then
					prisoner2 = actor;
				end

				if actor.PresetName == "Ronin Soldier 6 Nopack" then
					prisoner3 = actor;
				end	
			end

			--Set everyone to sentry mode so they don't wander.
			for actor in MovableMan.Actors do
				-- Set all actors to sentry AI mode
					actor:SetControllerMode(Controller.CIM_AI, -1);
					actor.AIMode = Actor.AIMODE_SENTRY;
        		end

			--Switch to the player.  The computer thinks it's switched, but it actually isn't in control.
			self:SwitchToActor(playerActor, 0, Activity.TEAM_1);
		end
		
		--If the explosion is in effect, do the following.
		if generatornova == 1 then
		local btime = math.ceil(self.ExplosionTimer:LeftTillSimMS(self.boomtimer)/1000);
			--Display the time left only if it is above zero.
			if btime > 0 then
				FrameMan:SetScreenText(math.ceil(btime) .. " seconds left until explosion!", Activity.TEAM_1, 0, 1, false);
			end
			--Check if the explosion should happen.
			if btime < 0 then
				MovableMan:RemoveActor(playerActor);
				FrameMan:SetScreenText("Sir? SIR?! SIIIIIIIIIR!(MISSION FAILED)", Activity.TEAM_1, 0, 5000, false);
				self.explode = 1;
				self.WinnerTeam = Activity.TEAM_2;
				
				AudioMan:ClearMusicQueue();
				AudioMan:PlayMusic("Base.rte/Music/dBSoundworks/udiedfinal.ogg", 0, -1);
				ActivityMan:EndActivity(); -- Game over
			end
		end

		--Update the timer.
		self.timer = self.timer + 1;
		local player = 0;
        	local team = Activity.TEAM_1;
	
		--Make sure the focus is always on playerActor.
		if MovableMan:IsActor(playerActor) then
			if (self:GetControlledActor(Activity.TEAM_1) == nil) then
				self:SwitchToActor(playerActor, 0, Activity.TEAM_1);
			else
				local pid = self:GetControlledActor(Activity.TEAM_1).ID;
				if playerActor.ID ~= pid then
					self:SwitchToActor(playerActor, player, team);
				end
			end
		end	

		-- This should only happen when the player is dead, because there is no playerActor.  Also check if the explosion has already happened, because then respawning would be a bit strange.
		if ((not MovableMan:IsActor(playerActor))) and self.explode == 0 then
			self.playerswitched = false;
			self.Lives = self.Lives - 1;
			--If the ship has come down already, it means the ship broke.  That means a game over.
			if shipdropped == 1 then
				self.Lives = 0;
				self.msg2 = 2;
				self.WinnerTeam = Activity.TEAM_2;
				ActivityMan:EndActivity(); -- Game over
			end
			--If the player is out of lives, then the game ends.
			if (self.Lives <= 0) then
				-- Game over
				FrameMan:SetScreenText("Despite your best efforts, you have died. The Ronin army is doomed. (MISSION FAILED)", Activity.TEAM_1, 0, 5000, false);
				
				
				AudioMan:ClearMusicQueue();
				AudioMan:PlayMusic("Base.rte/Music/dBSoundworks/udiedfinal.ogg", 0, -1);
				self.WinnerTeam = Activity.TEAM_2;
				ActivityMan:EndActivity(); -- Game over
			--Or, if there are still lives left, make another player.
			else
				-- Clear to make a new player
				playerActor = respawn();
			end
		end
		
		if (not self.playerswitched and MovableMan:IsActor(playerActor)) then
			-- Switch to the new player
			self:SwitchToActor(playerActor, 0, Activity.TEAM_1);
			self.playerswitched = true;
			self:ResetMessageTimer(Activity.TEAM_1);
            		FrameMan:ClearScreenText(Activity.TEAM_1);
			-- Display life counter
			if (self.Lives == 1) then
				FrameMan:SetScreenText(self.Lives .. " LIFE LEFT!", Activity.TEAM_1, 0, 1000, false);
			else
				FrameMan:SetScreenText(self.Lives .. " LIVES LEFT", Activity.TEAM_1, 0, 1000, false);
			end
		end

		--Once everything is spawned, check if the player has completed any objectives.
		if self.timer > 15 and MovableMan:IsActor(playerActor) then

			--Inside the prison cell.
			local area1 = SceneMan.Scene:GetArea("Obj1");
			--Bordering areas to the prison cell.
			local area2 = SceneMan.Scene:GetArea("Obj1Complete");

			--Check if the player is within the prison cell.  If they are, remind them of the current objective.
			if area1:IsInside(playerActor.Pos) and self.obj1 == 0 then
				self:ResetMessageTimer(Activity.TEAM_1);
        	    		FrameMan:ClearScreenText(Activity.TEAM_1);
				FrameMan:SetScreenText("Free yourself from the cell using the shovel you smuggled in from the workyard.", Activity.TEAM_1, 0, 1, false);
			end

			--Check if the player is within one of the environing areas to the prison cell.  If they are, tell them the next objective.
			if area2:IsInside(playerActor.Pos) and self.obj1 == 0 then
				self:ResetMessageTimer(Activity.TEAM_1);
       		     		FrameMan:ClearScreenText(Activity.TEAM_1);
				FrameMan:SetScreenText("Free your nearest comrade.", Activity.TEAM_1, 0, 5000, false);
				self.obj1 = 1;
			end

			--Check the first set of doors.  If they're not there, it means the player has broken out the first prisoner.  Set the message to explain the next objective.
			local door1open = 1;
			local doors = 0;

			if prisoner1free == 0 then
				for actor in MovableMan.Actors do
					--Here, we check the health of the door too.  The part of the door that actually counts as the door is the motor, which isn't technically dead until it explodes.
					--However, once the door is detached, it will begin to lose health.  So, if its health isn't equal to zero, then the door is dead.
					--This way we don't have to wait until the motor is gone.
					if actor.PresetName == "Door Rotate Short NoMove" and SceneMan.Scene:WithinArea("Prison Doors 1",actor.Pos) and actor.Health == 100 then
						doors = doors + 1;
					end
				end

				if doors == 2 then
					door1open = 0;
				end

				if door1open == 1 then
					self:ResetMessageTimer(Activity.TEAM_1);
        				FrameMan:ClearScreenText(Activity.TEAM_1);
					FrameMan:SetScreenText("Free your nearest comrade.", Activity.TEAM_1, 0, 5000, false);
					prisoner1free = 1;
				end
			end

			--Check the second set of doors.  If at least one is not there, it means the player has broken out the second prisoner.  Set the message to explain the next objective.
			local door2open = 1;

			if prisoner2free == 0 then
				for thing in MovableMan.Particles do
					if thing.PresetName == "Vent Cover" and SceneMan.Scene:WithinArea("Prison Doors 2",thing.Pos) then
						door2open = 0;
					end
				end

				if door2open == 1 then
					self:ResetMessageTimer(Activity.TEAM_1);
        				FrameMan:ClearScreenText(Activity.TEAM_1);
					FrameMan:SetScreenText("Destroy the signal jammer on top of the tower to contact home base.", Activity.TEAM_1, 0, 5000, false);
					prisoner2free = 1;
				end
			end

			--Check for the Signal Jammer.  If none are found, it's been broken.  Set the message to explain the next objective.
			local foundjammer = 0;
			if jammergone == 0 then
				for thing in MovableMan.Particles do
					if thing.PresetName == "Signal Jammer" then
						foundjammer = 1;
					end
				end

				if foundjammer == 0 then
					self:ResetMessageTimer(Activity.TEAM_1);
        				FrameMan:ClearScreenText(Activity.TEAM_1);
					FrameMan:SetScreenText("Your communicator batteries are dead. Free the last prisoner and use his.", Activity.TEAM_1, 0, 5000, false);
					jammergone = 1;		
				end
			end

			--Check the third set of doors.  If they're not there, it means the player has broken out the third prisoner.  Set the message to explain the next objective.
			local door3open = 1;
			local doorsb = 0;
			if prisoner3free == 0 then
				for thing in MovableMan.Particles do
					if thing.PresetName == "Vent Cover" and SceneMan.Scene:WithinArea("Prison Doors 3",thing.Pos) then
						doorsb = doorsb + 1;
					end
				end

				if doorsb > 0 then
					door3open = 0;
				end

				if door3open == 1 then
					self:ResetMessageTimer(Activity.TEAM_1);
        				FrameMan:ClearScreenText(Activity.TEAM_1);
					FrameMan:SetScreenText("Destroy the lower half of the power generator underneath the base!", Activity.TEAM_1, 0, 5000, false);
					prisoner3free = 1;
				end
			end

			--Check for a Generator Piece B (the lower half).  If none are found, it's been broken.  Set the message to explain the next objective.
			local foundgen = 0;
			if generatornova == 0 then
				for thing in MovableMan.Particles do
					if thing.PresetName == "Generator Piece B" then
						foundgen = 1;
					end
				end

				if foundgen == 0 then
					self:ResetMessageTimer(Activity.TEAM_1);
        				FrameMan:ClearScreenText(Activity.TEAM_1);
					FrameMan:SetScreenText("That generator is going to blow. Get out of there!", Activity.TEAM_1, 0, 5000, false);
					generatornova = 1;
					self.ExplosionTimer:Reset();
					--Move all the prisoners to the surface.
					if prisoner1free == 1 then
						prisoner1.Pos = Vector(816,144);
					end

					if prisoner2free == 1 then
						prisoner2.Pos = Vector(936,144);
					end

					if prisoner3free == 1 then
						prisoner3.Pos = Vector(852,144);
					end	
					--Set the music to something a bit more intense.
					AudioMan:ClearMusicQueue();
					AudioMan:PlayMusic("Prison Escape Ep1.rte/Music/Buzztone_Symphony.ogg", -1, -1);					
				end
			end
		end

		--Check if the player has entered checkpoints, and update the checkpoint variable if it has.
		if ((MovableMan:IsActor(playerActor))) then
			if self.check1:IsInside(playerActor.Pos) and checkpoint < 1 then
				checkpoint = 1;
			end
			if self.check2:IsInside(playerActor.Pos) and checkpoint < 2 then
				checkpoint = 2;
			end
			if self.check3:IsInside(playerActor.Pos) and checkpoint < 3 then
				checkpoint = 3;
			end

		--Check if the player is in the dropship zone, and if he is and it's not there yet, call it down.
			if generatornova == 1 and self.dropzone:IsInside(playerActor.Pos) and shipdropped == 0 then
				if jammergone == 1 then
					oldactor = playerActor;
					playerActor = CreateACDropShip("Drop Ship MK1 Nohurt");
					playerActor.Pos = Vector(840,0);
					MovableMan:AddActor(playerActor);
					self:ResetMessageTimer(Activity.TEAM_1);
        				FrameMan:ClearScreenText(Activity.TEAM_1);
					FrameMan:SetScreenText("Get everyone in the ship and get out STAT!", Activity.TEAM_1, 0, 5000, false);
					shipdropped = 1;
				else
					self:ResetMessageTimer(Activity.TEAM_1);
        				FrameMan:ClearScreenText(Activity.TEAM_1);
					FrameMan:SetScreenText("We can't send the ship down, the signal's jammed... Did you take out that jammer?", Activity.TEAM_1, 0, 5000, false);
				end
			end

		--Check if the player is attempting to escape through the vault next to the generator.  Ha ha, cheater, can't dig through 'em either.
			local doorarea = SceneMan.Scene:GetArea("Vault Door");
			if self.gensect:IsInside(playerActor.Pos) and self.rocksdropped == 0 then
				FrameMan:SetScreenText("We'll send you down some rein-- Uh oh.", Activity.TEAM_1, 0, 5000, false);
				for actor in MovableMan.Actors do
					if doorarea:IsInside(actor.Pos) then
						--Set the door's team to -1 so it doesn't count as a casualty.
						actor.Team = -1;
						actor:GibThis();
					end
				end
				--Make 5 boulders and a dropship and send them down the tube to block it.
				local bould1 = CreateMOSRotating("Boulder Gib A");
				local bould2 = CreateMOSRotating("Boulder Gib B");
				local bould3 = CreateMOSRotating("Boulder Gib C");
				local bould4 = CreateMOSRotating("Boulder Gib D");
				local bould5 = CreateMOSRotating("Boulder Gib E");
				local breakship = CreateACDropShip("Drop Ship MK1");
				--Make the ship's team -1 so it doesn't count as a casualty.
				breakship.Team = -1;
				bould1.Pos = Vector(1380,300);
				bould2.Pos = Vector(1380,350);
				bould3.Pos = Vector(1380,350);
				bould4.Pos = Vector(1380,350);
				bould5.Pos = Vector(1380,350);
				breakship.Pos = Vector(1380,252);
				MovableMan:AddParticle(bould1);
				MovableMan:AddParticle(bould2);
				MovableMan:AddParticle(bould3);
				MovableMan:AddParticle(bould4);
				MovableMan:AddParticle(bould5);
				MovableMan:AddParticle(breakship);
				breakship:GibThis();
				--If we don't set this, this entire block will be repeated infinitely so long as the player is within the drop area.  That lags a lot.
				self.rocksdropped = 1;
			end
		end

		--Call the function that checks the inventory so we can update the weapon variables.
		checkinv();
		if MovableMan:IsActor(playerActor) then
			--If the player is alive, set its last position to this number.
			lastpos = playerActor.Pos;
			--Set the observation target, or what the game flies to when the game ends, to the player's last position.  Now do you see why we use the last position and not the current one?  If the game is over, there IS no player.
                	self:SetObservationTarget(lastpos, Activity.TEAM_1);
		end


		-- Objectives stuff--

    		-- Clear all objective markers, they get re-added each frame.  Otherwise there's a build-up of arrows and the game will lag.
    		self:ClearObjectivePoints();

    		if not (self.ActivityState == Activity.OVER) then
        		if prisoner1free == 0 and self.obj1 == 1 then
        		    	self:AddObjectivePoint("Free by destroying the doors!", prisoner1.AboveHUDPos+Vector(0,-8), Activity.TEAM_1, GameActivity.ARROWDOWN);
        		end

        		if prisoner2free == 0 and prisoner1free == 1 then
        		    	self:AddObjectivePoint("Free by destroying the vent cover!", prisoner2.AboveHUDPos+Vector(0,-8), Activity.TEAM_1, GameActivity.ARROWDOWN);
        		end

        		if jammergone == 0 and prisoner2free == 1 then
        		    	self:AddObjectivePoint("Destroy the signal jammer and open communications!", Vector(420,48), Activity.TEAM_1, GameActivity.ARROWDOWN);
        		end

        		if prisoner3free == 0 and jammergone == 1 then
        		    	self:AddObjectivePoint("Free by destroying the vent covers!", prisoner3.AboveHUDPos+Vector(0,-8), Activity.TEAM_1, GameActivity.ARROWDOWN);
        		end

        		if generatornova == 0 and prisoner3free == 1 then
        		    	self:AddObjectivePoint("Destroy the lower half of the generator!", Vector(1074,970), Activity.TEAM_1, GameActivity.ARROWDOWN);
        		end

				if generatornova == 1 and shipdropped == 0 then
					self:AddObjectivePoint("Get out of the base!", Vector(840,132), Activity.TEAM_1, GameActivity.ARROWDOWN);
				end

				if shipdropped == 1 then
					if MovableMan:IsActor(oldactor) then
						self:AddObjectivePoint("Pick up!", oldactor.AboveHUDPos+Vector(0,-8), Activity.TEAM_1, GameActivity.ARROWDOWN);
					end
					if MovableMan:IsActor(prisoner1) then
						self:AddObjectivePoint("Pick up!", prisoner1.Pos+Vector(0,8), Activity.TEAM_1, GameActivity.ARROWUP);
					end
					if MovableMan:IsActor(prisoner2) then
						self:AddObjectivePoint("Pick up!", prisoner2.AboveHUDPos+Vector(0,-8), Activity.TEAM_1, GameActivity.ARROWDOWN);
					end
					if MovableMan:IsActor(prisoner3) then
						self:AddObjectivePoint("Pick up!", prisoner3.Pos+Vector(0,8), Activity.TEAM_1, GameActivity.ARROWUP);
					end
					if playerActor:HasObject("Dafred") and playerActor:HasObject("Ronin Soldier 4 Nopack") and playerActor:HasObject("Ronin Soldier 5 Nopack") and playerActor:HasObject("Ronin Soldier 6 Nopack") then
						self:AddObjectivePoint("Escape into the atmosphere!", Vector(playerActor.Pos.X,0), Activity.TEAM_1, GameActivity.ARROWUP);
					end
				end
			end

        	-- Sort the objective points.
        	self:YSortObjectivePoints();
	else
        if glowpos then
            SceneMan:SetScrollTarget(glowpos, 1, false, Activity.PLAYER_1);
        end
    end
end

function PrisonEscapeMission:CraftEnteredOrbit()
	--Check if a craft has returned.  Check if it has the right contents.  If it does, mission complete!
	if shipdropped == 1 then
		if self.OrbitedCraft:HasObject("Dafred") and self.OrbitedCraft:HasObject("Ronin Soldier 4 Nopack") and self.OrbitedCraft:HasObject("Ronin Soldier 5 Nopack") and self.OrbitedCraft:HasObject("Ronin Soldier 6 Nopack") then
			self.explode = 1;
			self.WinnerTeam = Activity.TEAM_1;
			ActivityMan:EndActivity();
		else
			--And if it DOESN'T have the right contents, mission failed!
			self.msg2 = 1;
			self.explode = 1;
			self.WinnerTeam = Activity.TEAM_2;
			ActivityMan:EndActivity();
		end
	end
end